@@ -181,13 +181,13 @@ soup_session_init (SoupSession *session)
 
 	priv->features_cache = g_hash_table_new (NULL, NULL);
 
-	auth_manager = g_object_new (SOUP_TYPE_AUTH_MANAGER_NTLM,
-				     SOUP_AUTH_MANAGER_NTLM_USE_NTLM, FALSE,
-				     NULL);
+	auth_manager = g_object_new (SOUP_TYPE_AUTH_MANAGER_NTLM, NULL);
 	g_signal_connect (auth_manager, "authenticate",
 			  G_CALLBACK (auth_manager_authenticate), session);
-	soup_auth_manager_add_type (auth_manager, SOUP_TYPE_AUTH_BASIC);
-	soup_auth_manager_add_type (auth_manager, SOUP_TYPE_AUTH_DIGEST);
+	soup_session_feature_add_feature (SOUP_SESSION_FEATURE (auth_manager),
+					  SOUP_TYPE_AUTH_BASIC);
+	soup_session_feature_add_feature (SOUP_SESSION_FEATURE (auth_manager),
+					  SOUP_TYPE_AUTH_DIGEST);
 	soup_session_add_feature (session, SOUP_SESSION_FEATURE (auth_manager));
 	g_object_unref (auth_manager);
 
@@ -489,6 +489,14 @@ soup_session_class_init (SoupSessionClass *session_class)
 				   "Connection lifetime when idle",
 				   0, G_MAXUINT, 0,
 				   G_PARAM_READWRITE));
+	/**
+	 * SoupSession:use-ntlm:
+	 *
+	 * Whether or not to use NTLM authentication.
+	 *
+	 * Deprecated: use soup_session_add_feature_by_type() with
+	 * #SOUP_TYPE_AUTH_NTLM.
+	 **/
 	/**
 	 * SOUP_SESSION_USE_NTLM:
 	 *
@@ -849,9 +857,10 @@ set_property (GObject *object, guint prop_id,
 	case PROP_USE_NTLM:
 		feature = soup_session_get_feature (session, SOUP_TYPE_AUTH_MANAGER_NTLM);
 		if (feature) {
-			g_object_set_property (G_OBJECT (feature),
-					       SOUP_AUTH_MANAGER_NTLM_USE_NTLM,
-					       value);
+			if (g_value_get_boolean (value))
+				soup_session_feature_add_feature (feature, SOUP_TYPE_AUTH_NTLM);
+			else
+				soup_session_feature_remove_feature (feature, SOUP_TYPE_AUTH_NTLM);
 		} else
 			g_warning ("Trying to set use-ntlm on session with no auth-manager");
 		break;
@@ -956,11 +965,9 @@ get_property (GObject *object, guint prop_id,
 		break;
 	case PROP_USE_NTLM:
 		feature = soup_session_get_feature (session, SOUP_TYPE_AUTH_MANAGER_NTLM);
-		if (feature) {
-			g_object_get_property (G_OBJECT (feature),
-					       SOUP_AUTH_MANAGER_NTLM_USE_NTLM,
-					       value);
-		} else
+		if (feature)
+			g_value_set_boolean (value, soup_session_feature_has_feature (feature, SOUP_TYPE_AUTH_NTLM));
+		else
 			g_value_set_boolean (value, FALSE);
 		break;
 	case PROP_SSL_CA_FILE:
@@ -1626,14 +1633,8 @@ cancel_message (SoupSession *session, SoupMessage *msg, guint status_code)
 	item = soup_message_queue_lookup (priv->queue, msg);
 	g_return_if_fail (item != NULL);
 
-	if (item->cancellable)
-		g_cancellable_cancel (item->cancellable);
-
 	soup_message_set_status (msg, status_code);
-	if (soup_message_io_in_progress (msg))
-		soup_message_io_finished (msg);
-	else
-		item->state = SOUP_MESSAGE_FINISHING;
+	g_cancellable_cancel (item->cancellable);
 
 	soup_message_queue_item_unref (item);
 }
@@ -1802,12 +1803,19 @@ soup_session_add_feature (SoupSession *session, SoupSessionFeature *feature)
 /**
  * soup_session_add_feature_by_type:
  * @session: a #SoupSession
- * @feature_type: the #GType of a class that implements #SoupSessionFeature
+ * @feature_type: a #GType
  *
- * Creates a new feature of type @feature_type and adds it to
- * @session. You can use this instead of soup_session_add_feature() in
- * the case wher you don't need to customize the new feature in any
- * way. You can also add a feature to the session at construct time by
+ * If @feature_type is the type of a class that implements
+ * #SoupSessionFeature, this creates a new feature of that type and
+ * adds it to @session as with soup_session_add_feature(). You can use
+ * this when you don't need to customize the new feature in any way.
+ *
+ * If @feature_type is not a #SoupSessionFeature type, this gives
+ * each existing feature on @session the chance to accept @feature_type
+ * as a "subfeature". This can be used to add new #SoupAuth types,
+ * for instance.
+ *
+ * You can also add a feature to the session at construct time by
  * using the %SOUP_SESSION_ADD_FEATURE_BY_TYPE property.
  *
  * Since: 2.24
@@ -1815,14 +1823,24 @@ soup_session_add_feature (SoupSession *session, SoupSessionFeature *feature)
 void
 soup_session_add_feature_by_type (SoupSession *session, GType feature_type)
 {
-	SoupSessionFeature *feature;
-
 	g_return_if_fail (SOUP_IS_SESSION (session));
-	g_return_if_fail (g_type_is_a (feature_type, SOUP_TYPE_SESSION_FEATURE));
 
-	feature = g_object_new (feature_type, NULL);
-	soup_session_add_feature (session, feature);
-	g_object_unref (feature);
+	if (g_type_is_a (feature_type, SOUP_TYPE_SESSION_FEATURE)) {
+		SoupSessionFeature *feature;
+
+		feature = g_object_new (feature_type, NULL);
+		soup_session_add_feature (session, feature);
+		g_object_unref (feature);
+	} else {
+		SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
+		GSList *f;
+
+		for (f = priv->features; f; f = f->next) {
+			if (soup_session_feature_add_feature (f->data, feature_type))
+				return;
+		}
+		g_warning ("No feature manager for feature of type '%s'", g_type_name (feature_type));
+	}
 }
 
 /**
@@ -1853,7 +1871,7 @@ soup_session_remove_feature (SoupSession *session, SoupSessionFeature *feature)
 /**
  * soup_session_remove_feature_by_type:
  * @session: a #SoupSession
- * @feature_type: the #GType of a class that implements #SoupSessionFeature
+ * @feature_type: a #GType
  *
  * Removes all features of type @feature_type (or any subclass of
  * @feature_type) from @session. You can also remove standard features
@@ -1871,12 +1889,21 @@ soup_session_remove_feature_by_type (SoupSession *session, GType feature_type)
 	g_return_if_fail (SOUP_IS_SESSION (session));
 
 	priv = SOUP_SESSION_GET_PRIVATE (session);
-restart:
-	for (f = priv->features; f; f = f->next) {
-		if (G_TYPE_CHECK_INSTANCE_TYPE (f->data, feature_type)) {
-			soup_session_remove_feature (session, f->data);
-			goto restart;
+
+	if (g_type_is_a (feature_type, SOUP_TYPE_SESSION_FEATURE)) {
+	restart:
+		for (f = priv->features; f; f = f->next) {
+			if (G_TYPE_CHECK_INSTANCE_TYPE (f->data, feature_type)) {
+				soup_session_remove_feature (session, f->data);
+				goto restart;
+			}
 		}
+	} else {
+		for (f = priv->features; f; f = f->next) {
+			if (soup_session_feature_remove_feature (f->data, feature_type))
+				return;
+		}
+		g_warning ("No feature manager for feature of type '%s'", g_type_name (feature_type));
 	}
 }
 
